local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local C = ShockAndAwe.constants -- Defined in ShockAndAwe LUA no locale needed.

ShockAndAwe.fonteffects = {
	["none"] = L["none"],
	["OUTLINE"] = L["OUTLINE"],
	["THICKOUTLINE"] = L["THICKOUTLINE"],
	["MONOCHROME"] = L["MONOCHROME"],
}

function ShockAndAwe:GetOptions()
	local options = { 
		name = "ShockAndAwe",
		handler = ShockAndAwe,
		type='group',
		childGroups ='tree',
		args = {
			frames = {
				name = L["Frame Options"],
				type = 'group',
				order = 1,
				args = {
					width = {
						type = 'range',
						name = L["Bar Width"],
						desc = L["help_width"],
						min = 100,
						max = 500,
						step = 10,
						get = "GetWidth",
						set = "SetWidth",
						order = 32,
					},
					scale = {
						type = 'range',
						name = L["Bar Scale"],
						desc = L["help_scale"],
						min = 0.25,
						max = 3.00,
						step = 0.05,
						get = "GetScale",
						set = "SetScale",
						order = 33,
					},
					resetbars = {
						type = 'execute',
						name = L["Reset Bars"],
						desc = L["help_reset"],
						func = "ResetBars",
						order = 34,
					},
					resetuptime = {
						type = 'execute',
						name = L["Reset Uptime"],
						desc = L["help_reset_uptime"],
						func = "ResetUptime",
						order = 35,
					},
					resetpriority = {
						type = 'execute',
						name = L["Reset Priority"],
						desc = L["help_reset_priority"],
						func = "ResetPriority",
						order = 36,
					},
				},
			},
			bars = {
				name = L["Timer Bars"],
				type = 'group',
				order = 2,
				args = {
					disable = {
						type = 'toggle',
						name = L["Disable out of combat"],
						desc = L["help_disable"],
						get = "disableQuery",
						set = "Activatedisable",
						order = 1,
						},
					icons = {
						type = 'toggle',
						name = L["Show bar icons"],
						desc = L["help_baricons"],
						get = "bariconsQuery",
						set = "ActivateBarIcons",
						order = 2,
						},
					break1 = {
						type = 'header',
						name = L["Bars Section"],
						dialogHidden = true,
						order = 10,
					},
					feralspiritbar = {
						type = 'toggle',
						name = L["Feral Spirit Bar"],
						desc = L["help_feralspirit"],
						get = "feralspiritQuery",
						set = "ActivateFeralSpirit",
						order = 11,
						},
					shieldbar = {
						type = 'toggle',
						name = L["Shield Bar"],
						desc = L["help_shield"],
						get = "shieldQuery",
						set = "ActivateShield",
						order = 11,
						},
					maelstrom = {
						type = 'toggle',
						name = L["Maelstrom Bar"],
						desc = L["help_maelstrom"],
						get = "maelstromQuery",
						set = "ActivateMaelstrom",
						order = 12,
						},
					SS = {
						type = 'toggle',
						name = L["Stormstrike Bar"],
						desc = L["help_ss"],
						get = "SSQuery",
						set = "ActivateSS",
						order = 13,
					},
					WF = {
						type = 'toggle',
						name = L["Windfury Bar"],
						desc = L["help_wf"],
						get = "WFQuery",
						set = "ActivateWF",
						order = 14,
					},
					shock = {
						type = 'toggle',
						name = L["Shock Bar"],
						desc = L["help_shock"],
						get = "shockQuery",
						set = "ActivateShock",
						order = 15,
					},
					gcd = {
						type = 'toggle',
						name = L["GCD"],
						desc = L["help_gcd"],
						get = "gcdQuery",
						set = "ActivateGCD",
						order = 16,
					},
					lavalash = {
						type = 'toggle',
						name = C["Lava Lash"],
						desc = L["help_lavalash"],
						get = "lavalashQuery",
						set = "ActivateLavaLash",
						order = 17,
					},
					break4 = {
						type = 'header',
						name = L["Misc Options"],
						dialogHidden = true,
						order = 40,
					},
					activeshield = {
						type = 'toggle',
						name = L["Show Active Shields"],
						desc = L["help_activeshields"],
						get = "activeShieldsQuery",
						set = "ActivateActiveShields",
						order = 41,
					},
					arena = {
						type = 'toggle',
						name = L["4 piece arena bonus"],
						desc = L["help_arena"],
						get = "arenaQuery",
						set = "ActivateArena",
						order = 42,
					},
					text = {
						type = 'toggle',
						name = L["Text Display"],
						desc = L["help_textonbars"],
						get = "textOnBarsQuery",
						set = "ActivateTextOnBars",
						order = 43,
					},
					mw4sound = {
						type = 'toggle',
						name = L["Maelstrom Sound 4"],
						desc = L["help_mw4soundplay"],
						get = "mw4soundplayQuery",
						set = "ActivateMW4sound",
						order = 44,
					},
					mw5sound = {
						type = 'toggle',
						name = L["Maelstrom Sound 5"],
						desc = L["help_mw5soundplay"],
						get = "mw5soundplayQuery",
						set = "ActivateMW5sound",
						order = 45,
					},
					mwflash = {
						type = 'toggle',
						name = L["Maelstrom Flash"],
						desc = L["help_mw5flash"],
						get = "mw5flashQuery",
						set = "ActivateMW5flash",
						order = 46,
					},
				},
			},
			barcolours = {
				name = L["Bar Colours"],
				type = 'group',
				order = 3, 
				args = {
					break1 = {
						type = 'header',
						name = L["Shields"],
						dialogHidden = true,
						order = 1,
					},
					watershield = {
						type = 'color',
						name = L["Water Shield Colour"],
						desc = L["colWaterShield"],
						get = "getWaterShieldColour",
						set = "setWaterShieldColour",
						hasAlpha = true,
						order = 2,
					},
					lightningshield = {
						type = 'color',
						name = L["Lightning Shield Colour"],
						desc = L["colLightningShield"],
						get = "getLightningShieldColour",
						set = "setLightningShieldColour",
						hasAlpha = true,
						order = 3,
					},
					earthshield = {
						type = 'color',
						name = L["Earth Shield Colour"],
						desc = L["colEarthShield"],
						get = "getEarthShieldColour",
						set = "setEarthShieldColour",
						hasAlpha = true,
						order = 4,
					},
					noshield = {
						type = 'color',
						name = L["Shield Down Colour"],
						desc = L["colNoShield"],
						get = "getNoShieldColour",
						set = "setNoShieldColour",
						hasAlpha = false,
						order = 5,
					},
					break2 = {
						type = 'header',
						name = L["Shocks"],
						dialogHidden = true,
						order = 6,
					},
					flameshock = {
						type = 'color',
						name = L["Flame Shock Colour"],
						desc = L["colFlameShock"],
						get = "getFlameShockColour",
						set = "setFlameShockColour",
						hasAlpha = true,
						order = 7,
					},
					earthshock = {
						type = 'color',
						name = L["Earth Shock Colour"],
						desc = L["colEarthShock"],
						get = "getEarthShockColour",
						set = "setEarthShockColour",
						hasAlpha = true,
						order = 8,
					},
					frostshock = {
						type = 'color',
						name = L["Frost Shock Colour"],
						desc = L["colFrostShock"],
						get = "getFrostShockColour",
						set = "setFrostShockColour",
						hasAlpha = true,
						order = 9,
					},
					windshock = {
						type = 'color',
						name = L["Wind Shock Colour"],
						desc = L["colWindShock"],
						get = "getWindShockColour",
						set = "setWindShockColour",
						hasAlpha = true,
						order = 10,
					},
					break3 = {
						type = 'header',
						name = L["Maelstrom Bar"],
						dialogHidden = true,
						order = 11,
					},
					maelstrom = {
						type = 'color',
						name = L["Maelstrom Bar Colour"],
						desc = L["colMaelstrom"],
						get = "getMaelstromColour",
						set = "setMaelstromColour",
						hasAlpha = false,
						order = 12,
					},
					msalpha = {
						type = 'range',
						name = L["Maelstrom Alpha"],
						desc = L["colMaelstromAlpha"],
						min = 0,
						max = 1,
						isPercent = true,
						get = "GetMSalpha",
						set = "SetMSalpha",
						order = 13,
					},
					msalphaFull = {
						type = 'range',
						name = L["Maelstrom Full Alpha"],
						desc = L["colMaelstromFullAlpha"],
						min = 0,
						max = 1,
						isPercent = true,
						get = "GetMSalphaFull",
						set = "SetMSalphaFull",
						order = 14,
					},
					break4 = {
						type = 'header',
						name = L["Other Bars"],
						dialogHidden = true,
						order = 15,
					},
					stormstrike = {
						type = 'color',
						name = L["Stormstrike Bar Colour"],
						desc = L["colStormstrike"],
						get = "getStormstrikeColour",
						set = "setStormstrikeColour",
						hasAlpha = true,
						order = 16,
					},
					windfury = {
						type = 'color',
						name = L["Windfury Bar Colour"],
						desc = L["colWindfury"],
						get = "getWindfuryColour",
						set = "setWindfuryColour",
						hasAlpha = true,
						order = 17,
					},
					lavalash = {
						type = 'color',
						name = L["Lava Lash Bar Colour"],
						desc = L["colLavaLash"],
						get = "getLavaLashColour",
						set = "setLavaLashColour",
						hasAlpha = true,
						order = 18,
					},
					feralspirit = {
						type = 'color',
						name = L["Feral Spirit Bar Colour"],
						desc = L["colFeralSpirit"],
						get = "getFeralSpiritColour",
						set = "setFeralSpiritColour",
						hasAlpha = true,
						order = 19,
					},
					feralspiritCD = {
						type = 'color',
						name = L["Feral Spirit CD Bar Colour"],
						desc = L["colFeralSpiritCD"],
						get = "getFeralSpiritCDColour",
						set = "setFeralSpiritCDColour",
						hasAlpha = true,
						order = 20,
					},
				},
			},
			priority = {
				name = L["Priority Frame"],
				type = 'group',
				order = 4, 
				args = {
					priority = {
						type = 'toggle',
						name = L["Priority Frame"],
						desc = L["help_priority"],
						get = "priorityQuery",
						set = "ActivatePriority",
						order = 21,
					},
					title = {
						type = 'toggle',
						name = L["Display frame title"],
						get = "priorityTitleQuery",
						set = "ActivatePriorityTitle",
						order = 22,
					},
					combopoints = {
						type = 'toggle',
						name = L["Combo Points"],
						desc = L["help_combopoints"],
						get = "combopointsQuery",
						set = "ActivateComboPoints",
						order = 23,
					},
					windshock = {
						type = 'toggle',
						name = C["Wind Shock"],
						desc = L["help_windshock"],
						get = "windshockQuery",
						set = "ActivateWindShock",
						order = 24,
					},
					worldbossonly = {
						type = 'toggle',
						name = L["World Boss Only"],
						desc = L["help_worldbossonly"],
						get = "worldbossQuery",
						set = "ActivateWorldBoss",
						order = 25,
					},
					priority1 = {
						type = 'select',
						name = L["First Priority"],
						get = "GetFirstPriority",
						set = "SetFirstPriority",
						values = self.priorityTable.name,
						order = 26,
					},
					priority2 = {
						type = 'select',
						name = L["Second Priority"],
						get = "GetSecondPriority",
						set = "SetSecondPriority",
						values = self.priorityTable.name,
						order = 27,
					},
					priority3 = {
						type = 'select',
						name = L["Third Priority"],
						get = "GetThirdPriority",
						set = "SetThirdPriority",
						values = self.priorityTable.name,
						order = 28,
					},
					priority4 = {
						type = 'select',
						name = L["Fourth Priority"],
						get = "GetFourthPriority",
						set = "SetFourthPriority",
						values = self.priorityTable.name,
						order = 29,
					},
					priority5 = {
						type = 'select',
						name = L["Fifth Priority"],
						get = "GetFifthPriority",
						set = "SetFifthPriority",
						values = self.priorityTable.name,
						order = 30,
					},
					priority6 = {
						type = 'select',
						name = L["Sixth Priority"],
						get = "GetSixthPriority",
						set = "SetSixthPriority",
						values = self.priorityTable.name,
						order = 31,
					},
					threat = {
						type = 'range',
						name = L["Threat Threshold"],
						desc = L["help_threat"],
						min = 75,
						max = 110,
						step = 1,
						get = "GetThreatThreshold",
						set = "SetThreatThreshold",
						order = 32,
					},
					cooldown = {
						type = 'range',
						name = L["Cooldown Threshold"],
						desc = L["help_cooldown"],
						min = 0,
						max = 1.5,
						step = 0.05,
						get = "GetCooldownThreshold",
						set = "SetCooldownThreshold",
						order = 33,
					},
					fstimeleft = {
						type = 'range',
						name = L["Flame Shock Time Left"],
						desc = L["help_fstimeleft"],
						min = 0,
						max = 6,
						step = 0.1,
						get = "GetFSTimeLeft",
						set = "SetFSTimeLeft",
						order = 34,
					},
--~ 					totemtimeleft = {
--~ 						type = 'range',
--~ 						name = L["Totem Time Left"],
--~ 						desc = L["help_totemtimeleft"],
--~ 						min = 0,
--~ 						max = 300,
--~ 						step = 5,
--~ 						get = "GetTotemTimeLeft",
--~ 						set = "SetTotemTimeLeft",
--~ 						order = 35,
--~ 					},
					hwhealth = {
						type = 'range',
						name = L["Healing Wave Health%"],
						desc = L["help_hwhealth"],
						min = 1,
						max = 100,
						step = 1,
						get = "GetHWHealth",
						set = "SetHWHealth",
						order = 35,
					},
					chingroup = {
						type = 'toggle',
						name = L["Chain Heal in group"],
						desc = L["help_chingroup"],
						get = "GetCHInGroup",
						set = "SetCHInGroup",
						order = 36,
					},
					srmana = {
						type = 'range',
						name = L["Shamanistic Rage Level%"],
						desc = L["help_srmana"],
						min = 0,
						max = 50,
						step = 1,
						get = "GetSRMana",
						set = "SetSRMana",
						order = 37,
					},
					wsmana = {
						type = 'range',
						name = L["Water Shield Level%"],
						desc = L["help_wsmana"],
						min = 0,
						max = 100,
						step = 1,
						get = "GetWSMana",
						set = "SetWSMana",
						order = 38,
					},
					shieldorbs = {
						type = 'range',
						name = L["Shield orbs left"],
						desc = L["help_shieldorbs"],
						min = 0,
						max = 5,
						step = 1,
						get = "GetShieldOrbs",
						set = "SetShieldOrbs",
						order = 39,
					},
				},
			},
			warning = {
				name = L["Warning Options"],
				type = 'group',
				order = 5, 
				args = {
					show = {
						type = 'toggle',
						name = L["Use Warning Frame"],
						desc = L["help_warningframe"],
						get = "WarningFrameQuery",
						set = "ActivateWarningFrame",
						order = 1,
					},
					shield = {
						type = 'toggle',
						name = L["Shield Warning"],
						desc = L["help_shieldwarning"],
						get = "shieldWarnQuery",
						set = "ActivateShieldWarn",
						order = 2,
					},
					weapons = {
						type = 'toggle',
						name = L["Weapon Inbue Warning"],
						desc = L["help_weaponwarning"],
						get = "weaponWarnQuery",
						set = "ActivateWeaponWarn",
						order = 3,
					},
					range = {
						type = 'toggle',
						name = L["Out of range"],
						desc = L["help_outofrange"],
						get = "rangeWarnQuery",
						set = "ActivateRangeWarn",
						order = 4,
					},
					mainhand = {
						type = 'select',
						name = L["Main Hand Imbue"],
						get = "GetMainHandImbue",
						set = "SetMainHandImbue",
						values = ShockAndAwe.bindings.imbues,
						order = 5,
					},
					offhand = {
						type = 'select',
						name = L["Off Hand Imbue"],
						get = "GetOffHandImbue",
						set = "SetOffHandImbue",
						values = ShockAndAwe.bindings.imbues,
						order = 6,
					},
					colour = {
						type = 'color',
						name = L["Warning Msg Colour"],
						desc = L["colWarningMessage"],
						get = "getWarningColour",
						set = "setWarningColour",
						hasAlpha = true,
						order = 7,
					},
					duration = {
						type = 'range',
						name = L["Warning Message Duration"],
						min = 1,
						max = 10,
						step = .2,
						get = "GetWarningDuration",
						set = "SetWarningDuration",
						order = 8,
					},
--~ 					rebuff = {
--~ 						type = 'range',
--~ 						name = L["Weapon Rebuff time"],
--~ 						desc = L["help_weaponrebuff"],
--~ 						min = 0,
--~ 						max = 900,
--~ 						step = 10,
--~ 						get = "GetRebuffTime",
--~ 						set = "SetRebuffTime",
--~ 						order = 6,
--~ 					},
				},
			},
			media = {
				name = L["Media Options"],
				type = 'group',
				order = 6, 
				args = {
					texture = {
						type = 'select',
						name = L["Bar Texture"],
						get = "GetBarTexture",
						set = "SetBarTexture",
						values = self.textures,
						order = 1,
					},
					border = {
						type = 'select',
						name = L["Border Texture"],
						get = "GetBorderTexture",
						set = "SetBorderTexture",
						values = self.borders,
						order = 2,
					},
					barborder = {
						type = 'select',
						name = L["Bar Border Texture"],
						get = "GetBarBorderTexture",
						set = "SetBarBorderTexture",
						values = self.borders,
						order = 3,
					},
					barfont = {
						type = 'select',
						name = L["Bar Font"],
						get = "GetBarFont",
						set = "SetBarFont",
						values = self.fonts,
						order = 4,
					},
					barfonteffect = {
						name = L["Bar Font Effect"],
						type = "select",
						get = "GetBarFontEffect",
						set = "SetBarFontEffect",
						values = self.fonteffects,
						order = 5,
					},
					barfontsize = {
						type = 'range',
						name = L["Bar Font Size"],
						min = 6,
						max = 32,
						step = 1,
						get = "GetBarFontSize",
						set = "SetBarFontSize",
						order = 6,
					},
					msgfont = {
						type = 'select',
						name = L["Message Font"],
						get = "GetMsgFont",
						set = "SetMsgFont",
						values = self.fonts,
						order = 7,
					},
					msgfonteffect = {
						name = L["Message Font Effect"],
						type = "select",
						get = "GetMsgFontEffect",
						set = "SetMsgFontEffect",
						values = self.fonteffects,
						order = 8,
					},
					msgfontsize = {
						type = 'range',
						name = L["Message Font Size"],
						min = 6,
						max = 32,
						step = 1,
						get = "GetMsgFontSize",
						set = "SetMsgFontSize",
						order = 9,
					},
					mw4sound = {
						type = 'select',
						name = L["Maelstrom Sound 4"],
						get = "GetMW4sound",
						set = "SetMW4sound",
						values = self.sounds,
						order = 10,
					},
					mw5sound = {
						type = 'select',
						name = L["Maelstrom Sound 5"],
						get = "GetMW5sound",
						set = "SetMW5sound",
						values = self.sounds,
						order = 11,
					},
					mw5repeat = {
						type = 'range',
						name = L["Sound Repeat Interval"],
						desc = L["help_mw5repeat"],
						min = 1,
						max = 10,
						step = 0.1,
						get = "GetMW5repeat",
						set = "SetMW5repeat",
						order = 12,
					},
					shieldsound = {
						type = 'select',
						name = L["Shield Sound"],
						get = "GetShieldSound",
						set = "SetShieldSound",
						values = self.sounds,
						order = 13,
					},
				},
			},
			uptime = {
				name = L["Uptime Frame"],
				type = 'group',
				order = 7,
				args = {
					break1 = {
						type = 'header',
						name = L["Bar Colours"],
						dialogHidden = true,
						order = 1,
					},
					flurry = {
						type = 'color',
						name = L["Flurry Colour"],
						desc = L["colFlurry"],
						get = "getFlurryColour",
						set = "setFlurryColour",
						hasAlpha = true,
						order = 2,
					},					
					ed = {
						type = 'color',
						name = L["Elemental Devastation Colour"],
						desc = L["colED"],
						get = "getEDColour",
						set = "setEDColour",
						hasAlpha = true,
						order = 2,
					},					
					ur = {
						type = 'color',
						name = L["Unleashed Rage Colour"],
						desc = L["colUR"],
						get = "getURColour",
						set = "setURColour",
						hasAlpha = true,
						order = 2,
					},					
					break2 = {
						type = 'header',
						name = L["Misc Options"],
						dialogHidden = true,
						order = 10,
					},
					show = {
						type = 'toggle',
						name = L["Show Frame"],
						desc = L["help_showuptime"],
						get = "showUptimeQuery",
						set = "ActivateShowUptime",
						order = 11,
					},
					resetsession = {
						type = 'execute',
						name = L["Reset Session info"],
						desc = L["help_reset"],
						func = "InitialiseUptime",
						order = 13,
					},			
				},			
			},
			stats = {
				name = L["Stats Frame"],
				type = 'group',
				order = 8,
				args = {
					showWFtotals = {
						type = 'toggle',
						name = L["Show Windfury Totals"],
						desc = L["help_showwftotals"],
						get = "GetShowWFTotals",
						set = "SetShowWFTotals",
						order = 1,
					},
					wfcol = {
						type = 'color',
						name = L["Windfury Totals Colour"],
						desc = L["colWFTotals"],
						get = "getWFTotalsColour",
						set = "setWFTotalsColour",
						hasAlpha = true,
						order = 7,
					},
				},
			},				
			debug = {
				type = 'toggle',
				name = L["Debug mode"],
				desc = L["help_debug"],
				get = "debugQuery",
				set = "ActivateDebug",
				order = 9,
			},
			moveframes = {
				type = 'toggle',
				name = L["Move Frames"],
				desc = L["help_display"],
				get = "BarShowQuery",
				set = "ShowHideBars",
				order = 10,
			},
			testbar = {
				type = 'execute',
				name = L["Display Test Bar"],
				desc = L["help_testbar"],
				func = "WFProcBar",
				order = 11,
			},
			export = {
				type = 'execute',
				name = L["Export to Sim"],
				desc = L["help_export"],
				func = "ExportToSim",
				order = 12,
			},
			config = {
				type = 'execute',
				name = L["Configure Options"],
				desc = L["help_config"],
				func = "OpenConfig",
				order = 13,
			},
			news = {
				type = 'execute',
				name = L["Display News"],
				desc = L["help_news"],
				func = function() self:News(true); end,
				order = 14,
			},
			version = {
				type = 'execute',
				name = L["Version"],
				desc = L["help_version"],
				func = "DisplayVersion",
				order = 15,
			},
			help = {
				type = 'description',
				name = L["help"],
				guiHidden = true,
				order = 16,
			},
		},
	}
	return options
end

function ShockAndAwe:ActivateSS()
	self.db.char.ssshow = not self.db.char.ssshow
	self:CreateBaseFrame()
	if (self.db.char.ssshow) then
		self:Print(L["config_SS_on"])
	else
		self:Print(L["config_SS_off"])
	end
end

function ShockAndAwe:ActivateWF()
	self.db.char.wfshow = not self.db.char.wfshow
	self:CreateBaseFrame()
	if (self.db.char.wfshow) then
		self:Print(L["config_WF_on"])
	else
		self:Print(L["config_WF_off"])
	end
end

function ShockAndAwe:ActivateShock()
	self.db.char.shockshow = not self.db.char.shockshow
	self:CreateBaseFrame()
	if (self.db.char.shockshow) then
		self:Print(L["config_Shock_on"])
	else
		self:Print(L["config_Shock_off"])
	end
end

function ShockAndAwe:ActivateFeralSpirit()
	self.db.char.fsshow = not self.db.char.fsshow
	self:CreateBaseFrame()
	if (self.db.char.fsshow) then
		self:Print(L["config_feralspirit_on"])
	else
		self.frames["FeralSpirit"]:Hide()
		self:Print(L["config_feralspirit_off"])
	end
end

function ShockAndAwe:ActivateShield()
	self.db.char.shieldshow = not self.db.char.shieldshow
	self:CreateBaseFrame()
	if (self.db.char.shieldshow) then
		self:Print(L["config_Sheild_on"])
	else
		self.frames["Shield"]:Hide()
		self:Print(L["config_Sheild_off"])
	end
end

function ShockAndAwe:ActivateArena()
	self.db.char.arena = not self.db.char.arena
	self:SetTalentEffects()
	self:CreateBaseFrame()
	if (self.db.char.arena) then
		self:Print(L["config_Arena_on"])
	else
		self:Print(L["config_Arena_off"])
	end
end

function ShockAndAwe:ActivateActiveShields()
	self.db.char.activeshields = not self.db.char.activeshields
	self:UpdateShieldBar()
	if (self.db.char.activeshields) then
		self:Print(L["config_ActiveShields_on"])
	else
		self:Print(L["config_ActiveShields_off"])
	end
end

function ShockAndAwe:ActivateShowUptime()
	self.db.char.uptime.show = not self.db.char.uptime.show
	if (self.db.char.uptime.show) then
		self.UptimeFrame:Show()
		self:Print(L["config_Uptime_on"])
	else
		self.UptimeFrame:Hide()
		self:Print(L["config_Uptime_off"])
	end
end

function ShockAndAwe:ActivateMaelstrom()
	self.db.char.msshow = not self.db.char.msshow
	self:CreateBaseFrame()
	if (self.db.char.msshow) then
		self:MaelstromBar()
		self.db.char.msstacks = ShockAndAwe:GetMaelstromInfo()
		self:Print(L["config_MW_on"])
	else
		self.frames["Maelstrom"]:Hide()
		self:Print(L["config_MW_off"])
	end
end

function ShockAndAwe:ActivateLavaLash()
	self.db.char.llshow = not self.db.char.llshow
	self:CreateBaseFrame()
	if (self.db.char.llshow) then
		self:Print(L["config_LL_on"])
	else
		self.frames["LavaLash"]:Hide()
		self:Print(L["config_LL_off"])
	end
end

function ShockAndAwe:ActivateGCD()
	self.db.char.gcdshow = not self.db.char.gcdshow
	self:CreateBaseFrame()
	if (self.db.char.gcdshow) then
		self.frames["GCD"]:Show()
		self:Print(L["config_GCD_on"])
	else
		self.frames["GCD"]:Hide()
		self:Print(L["config_GCD_off"])
	end
end

function ShockAndAwe:Activatedisable()
	self.db.char.disablebars = not self.db.char.disablebars
	if (self.db.char.disablebars) then
		if not InCombatLockdown() then
			self.BaseFrame:Hide()
		end
		self:Print(L["config_disable_on"])
	else
		self.BaseFrame:Show()
		self:Print(L["config_disable_off"])
	end
end

function ShockAndAwe:ActivateBarIcons()
	self.db.char.showicons = not self.db.char.showicons
	if (self.db.char.showicons) then
		self:Print(L["config_baricons_on"])
	else
		self:Print(L["config_baricons_off"])
	end
end

function ShockAndAwe:ActivateDebug()
	self.db.char.debug = not self.db.char.debug
	if (self.db.char.debug) then
		self:Print(L["config_debug_on"])
	else
		self:Print(L["config_debug_off"])
	end
end

function ShockAndAwe:ActivateWindShock()
	self.db.char.windshockshow = not self.db.char.windshockshow
	if (self.db.char.windshockshow) then
		self:Print(L["config_WSicon_on"])
	else
		self:Print(L["config_WSicon_off"])
	end
end

function ShockAndAwe:ActivateComboPoints()
	self.db.char.priority.combopoints = not self.db.char.priority.combopoints
	if (self.db.char.priority.combopoints) then
		for index = 1, 5 do
			self.PriorityFrame.combo[index].frame:Show()
		end
		self:Print(L["config_combopoints_on"])
	else
		for index = 1, 5 do
			self.PriorityFrame.combo[index].frame:Hide()
		end
		self:Print(L["config_combopoints_off"])
	end
end

function ShockAndAwe:ActivateWorldBoss()
	self.db.char.priority.worldbossonly = not self.db.char.priority.worldbossonly
	if (self.db.char.priority.worldbossonly) then
		self:Print(L["config_worldboss_on"])
	else
		self:Print(L["config_worldboss_off"])
	end
end

function ShockAndAwe:ActivateTextOnBars()
	self.db.char.barstext = not self.db.char.barstext
	self:UpdateShieldBar()
	if (self.db.char.barstext) then
		self:Print(L["config_Barstext_on"])
	else
		self:Print(L["config_Barstext_off"])
	end
end

function ShockAndAwe:ActivateMW5sound()
	self.db.char.mw5soundplay = not self.db.char.mw5soundplay
	if (self.db.char.mw5soundplay) then
		self:Print(L["config_mw5soundplay_on"])
	else
		self:Print(L["config_mw5soundplay_off"])
	end
end

function ShockAndAwe:ActivateMW4sound()
	self.db.char.mw4soundplay = not self.db.char.mw4soundplay
	if (self.db.char.mw4soundplay) then
		self:Print(L["config_mw4soundplay_on"])
	else
		self:Print(L["config_mw4soundplay_off"])
	end
end

function ShockAndAwe:ActivateMW5flash()
	self.db.char.mw5flash = not self.db.char.mw5flash
	if (self.db.char.mw5flash) then
		self:Print(L["config_mw5flash_on"])
	else
		UIFrameFlashRemoveFrame(frames["Maelstrom"])
		ShockAndAwe:SetMaelstromAlpha(self.db.char.colours.msalpha)
		self:Print(L["config_mw5flash_off"])
	end
end

function ShockAndAwe:ShowHideBars()
	self.db.char.barshow = not self.db.char.barshow
	if self.db.char.barshow then
		self.PriorityFrame:EnableMouse(1)
		self.PriorityFrame:SetBackdropColor(0, 0, 0, 1)
		self.PriorityFrame:Show()
		self.UptimeFrame:EnableMouse(1)
		self.UptimeFrame:SetBackdropColor(0, 0, 0, 1)
		self.UptimeFrame:Show()
		self.msgFrame:EnableMouse(1)
		self.msgFrame:SetBackdropColor(0, 0, 0, 1)
		self.msgFrame:Show()
		self.BaseFrame:EnableMouse(1)
		self.BaseFrame:SetBackdropColor(0, 0, 0, 1);
		self.frames["Maelstrom"]:Show()
		self.frames["Stormstrike"]:Show()
		self.frames["Shock"]:Show()
		self.frames["Windfury"]:Show()
		self.frames["Shield"]:Show()
		self.frames["LavaLash"]:Show()
		self.frames["FeralSpirit"]:Show()
		self.frames["GCD"]:Show()
	else
		self.PriorityFrame:EnableMouse(0);
		self.PriorityFrame:SetBackdropColor(1, 1, 1, 0);
		self.BaseFrame:EnableMouse(0);
		self.BaseFrame:SetBackdropColor(1, 1, 1, 0);
		self.UptimeFrame:EnableMouse(0)
		self.UptimeFrame:SetBackdropColor(0, 0, 0, 0.3);
		self.msgFrame:EnableMouse(0)
		self.msgFrame:SetBackdropColor(1, 1, 1, 0);
		-- self.BaseFrame:Hide()
		self.frames["Maelstrom"]:Hide()
		self.frames["Stormstrike"]:Hide()
		self.frames["Shock"]:Hide()
		self.frames["Windfury"]:Hide()
		self.frames["LavaLash"]:Hide()
		self.frames["GCD"]:Hide()
		self:UpdateShieldBar()
		if self.db.char.priority.show and InCombatLockdown() then
			self.PriorityFrame:Show()
		else
			self.PriorityFrame:Hide()
		end
		if self.db.char.fsshow and InCombatLockdown() then
			self.frames["FeralSpirit"]:Show()
		else
			self.frames["FeralSpirit"]:Hide()
		end
		if self.db.char.uptime.show then
			self.UptimeFrame:Show()
		else
			self.UptimeFrame:Hide()
		end
	end
end

function ShockAndAwe:FinishedMoving(var, frame)
	local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint();
	var.point = point
	var.relativeTo = relativeTo
	var.relativePoint = relativePoint
	var.xOffset = xOffset
	var.yOffset = yOffset
end

function ShockAndAwe:SetShowWFTotals(info, newValue)
	self.db.char.stats.wfcalc = newValue
	if (self.db.char.stats.wfcalc) then
		self:Print(L["config_wfcalc_on"])
	else
		self:Print(L["config_wfcalc_off"])
	end
end

function ShockAndAwe:BarShowQuery()
	return self.db.char.barshow
end

function ShockAndAwe:SSQuery()
	return self.db.char.ssshow
end

function ShockAndAwe:WFQuery()
	return self.db.char.wfshow
end

function ShockAndAwe:shockQuery()
	return self.db.char.shockshow
end

function ShockAndAwe:gcdQuery()
	return self.db.char.gcdshow
end

function ShockAndAwe:windshockQuery()
	return self.db.char.windshockshow
end

function ShockAndAwe:combopointsQuery()
	return self.db.char.priority.combopoints
end

function ShockAndAwe:worldbossQuery()
	return self.db.char.priority.worldbossonly
end

function ShockAndAwe:feralspiritQuery()
	return self.db.char.fsshow
end

function ShockAndAwe:shieldQuery()
	return self.db.char.shieldshow
end

function ShockAndAwe:disableQuery()
	return self.db.char.disablebars
end

function ShockAndAwe:bariconsQuery()
	return self.db.char.showicons
end

function ShockAndAwe:arenaQuery()
	return self.db.char.arena
end

function ShockAndAwe:activeShieldsQuery()
	return self.db.char.activeshields
end

function ShockAndAwe:showUptimeQuery()
	return self.db.char.uptime.show
end

function ShockAndAwe:maelstromQuery()
	return self.db.char.msshow
end

function ShockAndAwe:lavalashQuery()
	return self.db.char.llshow
end

function ShockAndAwe:textOnBarsQuery()
    return self.db.char.barstext
end

function ShockAndAwe:mw5soundplayQuery()
    return self.db.char.mw5soundplay
end

function ShockAndAwe:mw4soundplayQuery()
    return self.db.char.mw4soundplay
end

function ShockAndAwe:mw5flashQuery()
    return self.db.char.mw5flash
end

function ShockAndAwe:debugQuery()
	return self.db.char.debug
end

function ShockAndAwe:GetBarTexture()
    return self.db.char.texture
end

function ShockAndAwe:GetBorderTexture()
    return self.db.char.border
end

function ShockAndAwe:GetBarBorderTexture()
    return self.db.char.barborder
end

function ShockAndAwe:GetBarFont()
    return self.db.char.barfont
end

function ShockAndAwe:GetBarFontSize()
    return self.db.char.barfontsize
end

function ShockAndAwe:GetBarFontEffect()
    return self.db.char.barfonteffect
end

function ShockAndAwe:GetMsgFont()
    return self.db.char.msgfont
end

function ShockAndAwe:GetMsgFontSize()
    return self.db.char.msgfontsize
end

function ShockAndAwe:GetMsgFontEffect()
    return self.db.char.msgfonteffect
end

function ShockAndAwe:GetWidth()
    return self.db.char.fWidth
end

function ShockAndAwe:GetScale()
    return self.db.char.scale
end

function ShockAndAwe:GetMSalpha()
    return self.db.char.colours.msalpha
end

function ShockAndAwe:GetMSalphaFull()
    return self.db.char.colours.msalphaFull
end

function ShockAndAwe:GetShieldSound()
    return self.db.char.shieldsoundname
end

function ShockAndAwe:GetMW5sound()
    return self.db.char.mw5soundname
end

function ShockAndAwe:GetMW5repeat()
    return self.db.char.mw5repeat
end

function ShockAndAwe:GetMW4sound()
    return self.db.char.mw4soundname
end

function ShockAndAwe:GetThreatThreshold()
    return self.db.char.threatThreshold
end

function ShockAndAwe:GetCooldownThreshold()
    return self.db.char.priority.cooldown
end

function ShockAndAwe:GetFSTimeLeft()
    return self.db.char.priority.fstimeleft
end

function ShockAndAwe:GetTotemTimeLeft()
    return self.db.char.priority.totemtimeleft
end

function ShockAndAwe:GetSRMana()
    return self.db.char.priority.srmana
end

function ShockAndAwe:GetWSMana()
    return self.db.char.priority.wsmana
end

function ShockAndAwe:GetShieldOrbs()
    return self.db.char.priority.shieldorbs
end

function ShockAndAwe:GetHWHealth()
    return self.db.char.priority.hwhealth
end

function ShockAndAwe:GetCHInGroup()
	return self.db.char.priority.chingroup 
end

function ShockAndAwe:GetRebuffTime()
    return self.db.char.warning.timeleft
end

function ShockAndAwe:GetShowWFTotals()
	return self.db.char.stats.wfcalc
end

function ShockAndAwe:SetCHInGroup()
	self.db.char.priority.chingroup = not self.db.char.priority.chingroup
	if (self.db.char.priority.chingroup) then
		self:Print(L["config_chingroup_on"])
	else
		self:Print(L["config_chingroup_off"])
	end
end

function ShockAndAwe:SetBarFont(info, newValue)
	self.db.char.barfont = newValue
	self:RedrawFrames()
end

function ShockAndAwe:SetBarFontSize(info, newValue)
	self.db.char.barfontsize = newValue
	self:RedrawFrames()
end

function ShockAndAwe:SetBarFontEffect(info, newValue)
	self.db.char.barfonteffect = newValue
	self:RedrawFrames()
end

function ShockAndAwe:SetMsgFont(info, newValue)
	self.db.char.msgfont = newValue
	self:CreateMsgFrame()
end

function ShockAndAwe:SetMsgFontSize(info, newValue)
	self.db.char.msgfontsize = newValue
	self:CreateMsgFrame()
end

function ShockAndAwe:SetMsgFontEffect(info, newValue)
	self.db.char.msgfonteffect = newValue
	self:CreateMsgFrame()
end

function ShockAndAwe:SetBarTexture(info, newValue)
	self.db.char.texture = newValue
	local barTexture = media:Fetch('statusbar', newValue)
	self.frames["FeralSpirit"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["Shield"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["Maelstrom"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["LavaLash"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["Stormstrike"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["Shock"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["Windfury"].statusbar:SetStatusBarTexture(barTexture)
	self.frames["GCD"].statusbar:SetStatusBarTexture(barTexture)
end

function ShockAndAwe:SetBorderTexture(info, newValue)
	self.db.char.border = newValue
	local newTexture = media:Fetch("border", self.db.char.border)
	if newTexture then
		self.frameBackdrop.edgeFile = newTexture
		self:RedrawFrames()
	else
		if self.db.char.debug then
			self:Print("border texture not found. Trying to set "..self.db.char.border)
		end
	end
end

function ShockAndAwe:SetBarBorderTexture(info, newValue)
	self.db.char.barborder = newValue
	local newTexture = media:Fetch("border", self.db.char.barborder)
	if newTexture then
		self.barBackdrop.edgeFile = newTexture
		self.frames["FeralSpirit"]:SetBackdrop(self.barBackdrop)
		self.frames["Shield"]:SetBackdrop(self.barBackdrop)
		self.frames["Maelstrom"]:SetBackdrop(self.barBackdrop)
		self.frames["LavaLash"]:SetBackdrop(self.barBackdrop)
		self.frames["Stormstrike"]:SetBackdrop(self.barBackdrop)
		self.frames["Shock"]:SetBackdrop(self.barBackdrop)
		self.frames["Windfury"]:SetBackdrop(self.barBackdrop)
		self.frames["GCD"]:SetBackdrop(self.barBackdrop)
		self:RedrawFrames()
	else
		if self.db.char.debug then
			self:Print("bar border texture not found. Trying to set "..self.db.char.barborder)
		end
	end
end

function ShockAndAwe:SetShieldSound(info, newValue)
	local newSound = media:Fetch("sound", newValue)
	self.db.char.shieldsoundname = newValue
	if newSound then
		self.db.char.shieldsound = newSound
		PlaySoundFile(newSound)
	else
		if self.db.char.debug then
			self:Print("Sound not found. Trying to set "..newSound)
		end
	end
end

function ShockAndAwe:SetMW5sound(info, newValue)
	local newSound = media:Fetch("sound", newValue)
	self.db.char.mw5soundname = newValue
	if newSound then
		self.db.char.mw5sound = newSound
		PlaySoundFile(newSound)
	else
		if self.db.char.debug then
			self:Print("Sound not found. Trying to set "..newSound)
		end
	end
end

function ShockAndAwe:SetMW5repeat(info, newValue)
	self.db.char.mw5repeat = newValue
end

function ShockAndAwe:SetMW4sound(info, newValue)
	local newSound = media:Fetch("sound", newValue)
	self.db.char.mw4soundname = newValue
	if newSound then
		self.db.char.mw4sound = newSound
		PlaySoundFile(newSound)
	else
		if self.db.char.debug then
			self:Print("Sound not found. Trying to set "..newSound)
		end
	end
end

function ShockAndAwe:SetThreatThreshold(info, newValue)
	self.db.char.threatThreshold = newValue
	self:Print(L["config_Threat"]..newValue)
end

function ShockAndAwe:SetCooldownThreshold(info, newValue)
	self.db.char.priority.cooldown = newValue
	self:Print(L["config_Cooldown"]..newValue)
end

function ShockAndAwe:SetFSTimeLeft(info, newValue)
	self.db.char.priority.fstimeleft = newValue
	self:Print(L["config_FSTimeLeft"]..newValue)
end

function ShockAndAwe:SetTotemTimeLeft(info, newValue)
	self.db.char.priority.totemtimeleft = newValue
	self:Print(L["config_TotemTimeLeft"]..newValue)
end

function ShockAndAwe:SetSRMana(info, newValue)
	self.db.char.priority.srmana = newValue
	self:Print(L["config_srmana"]..newValue)
end

function ShockAndAwe:SetShieldOrbs(info, newValue)
	self.db.char.priority.shieldorbs = newValue
	self:Print(L["config_shieldorbs"]..newValue)
end

function ShockAndAwe:SetWSMana(info, newValue)
	self.db.char.priority.wsmana = newValue
	self:Print(L["config_wsmana"]..newValue)
end

function ShockAndAwe:SetHWHealth(info, newValue)
	self.db.char.priority.hwhealth = newValue
	self:Print(L["config_hwhealth"]..newValue)
end

function ShockAndAwe:SetRebuffTime(info, newValue)
	self.db.char.warning.timeleft = newValue
end

function ShockAndAwe:SetWidth(info,newValue)
	local width = newValue
	self.db.char.fWidth = newValue
	self.BaseFrame:SetWidth(self.db.char.fWidth + self.db.char.fHeight / 7)
	
	self.frames["FeralSpirit"].statusbar:SetWidth(width - 6)
	self.frames["FeralSpirit"]:SetWidth(width)
	self.frames["FeralSpirit"].statusbar:SetValue(width - 6)
	self.frames["FeralSpirit"].spark:SetPoint("CENTER", self.frames["FeralSpirit"].statusbar,"LEFT",width - 3,-1)

	self.frames["Shield"].statusbar:SetWidth(width - 6)
	self.frames["Shield"]:SetWidth(width)
	self.frames["Shield"].statusbar:SetValue(width - 6)
	self.frames["Shield"].spark:SetPoint("CENTER", self.frames["Shield"].statusbar,"LEFT",width - 3,-1)

	self.frames["Maelstrom"].statusbar:SetWidth(width - 6)
	self.frames["Maelstrom"]:SetWidth(width)
	self.frames["Maelstrom"].statusbar:SetValue(width - 6)
	self.frames["Maelstrom"].spark:SetPoint("CENTER", self.frames["Maelstrom"].statusbar,"LEFT",width - 3,-1)
	
	width = self.db.char.fWidth * self.db.char.LLPercent
	self.frames["LavaLash"].statusbar:SetWidth(width - 6)
	self.frames["LavaLash"]:SetWidth(width)
	self.frames["LavaLash"].statusbar:SetValue(width - 6)
	self.frames["LavaLash"].spark:SetPoint("CENTER", self.frames["LavaLash"].statusbar,"LEFT",width - 3,-1)
	
	width = self.db.char.fWidth
	self.frames["GCD"].statusbar:SetWidth(width - 6)
	self.frames["GCD"]:SetWidth(width)
	self.frames["GCD"].statusbar:SetValue(width - 6)
	self.frames["GCD"].spark:SetPoint("CENTER", self.frames["GCD"].statusbar,"LEFT",width - 3,-1)
	self.GCDMin, self.GCDMax = self.frames["GCD"].statusbar:GetMinMaxValues()
	self.GCDWidth = self.frames["GCD"].statusbar:GetWidth()

	width = self.db.char.fWidth * self.db.char.SSPercent
	self.frames["Stormstrike"].statusbar:SetWidth(width - 6)
	self.frames["Stormstrike"]:SetWidth(width)
	self.frames["Stormstrike"].statusbar:SetValue(width - 6)
	self.frames["Stormstrike"].spark:SetPoint("CENTER", self.frames["Stormstrike"].statusbar,"LEFT",width - 3,-1)
	
	width = self.db.char.fWidth * self.db.char.ShockPercent
	self.frames["Shock"].statusbar:SetWidth(width - 6)
	self.frames["Shock"]:SetWidth(width)
	self.frames["Shock"].statusbar:SetValue(width - 6)
	self.frames["Shock"].spark:SetPoint("CENTER", self.frames["Shock"].statusbar,"LEFT",width - 3,-1)
	
	width = self.db.char.fWidth * self.db.char.WFProcPercent
	self.frames["Windfury"].statusbar:SetWidth(width - 6)
	self.frames["Windfury"]:SetWidth(width - 6)
	self.frames["Windfury"].spark:SetPoint("CENTER", self.frames["Windfury"].statusbar,"LEFT",width - 3,-1)
	self:Print("Frame width set to : "..width)	
end

function ShockAndAwe:SetScale(info,newValue)
	self.db.char.scale = newValue
	self.BaseFrame:SetScale(self.db.char.scale)
	self:Print("Scale set to : "..self.db.char.scale)
end

function ShockAndAwe:SetMSalpha(info,newValue)
	if self.db.char.msshow then
		self.db.char.colours.msalpha = newValue
		self:SetMaelstromAlpha(self.db.char.colours.msalpha)
		self:Print(L["config_MSAlpha_ooc"]..self.db.char.colours.msalpha)
	end
end

function ShockAndAwe:SetMSalphaFull(info,newValue)
	if self.db.char.msshow then
		self.db.char.colours.msalphaFull = newValue
		self:SetMaelstromAlpha(self.db.char.colours.msalphaFull)
		self:Print(L["config_MSAlpha_combat"]..self.db.char.colours.msalphaFull)
	end
end

function ShockAndAwe:SetMaelstromAlpha(alphaValue)
	self.frames["Maelstrom"]:SetBackdropColor(0, 0, 0, alphaValue * .2)
	self.frames["Maelstrom"]:SetBackdropBorderColor( 1, 1, 1, alphaValue)
	local colours = self.db.char.colours.maelstrom
	self.frames["Maelstrom"].statusbar:SetStatusBarColor( colours.r, colours.g, colours.b, alphaValue)
end

-----------------------------------------
-- Colour choices getter/setters
-----------------------------------------

function ShockAndAwe:getWaterShieldColour(info)
	local colours = self.db.char.colours.watershield
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setWaterShieldColour(info,r,g,b,a)
	self.db.char.colours.watershield.r = r
	self.db.char.colours.watershield.g = g
	self.db.char.colours.watershield.b = b
	self.db.char.colours.watershield.a = a
	self:UpdateShieldBar()
end

function ShockAndAwe:getLightningShieldColour(info)
	local colours = self.db.char.colours.lightningshield
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setLightningShieldColour(info,r,g,b,a)
	self.db.char.colours.lightningshield.r = r
	self.db.char.colours.lightningshield.g = g
	self.db.char.colours.lightningshield.b = b
	self.db.char.colours.lightningshield.a = a
	self:UpdateShieldBar()
end

function ShockAndAwe:getEarthShieldColour(info)
	local colours = self.db.char.colours.earthshield
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setEarthShieldColour(info,r,g,b,a)
	self.db.char.colours.earthshield.r = r
	self.db.char.colours.earthshield.g = g
	self.db.char.colours.earthshield.b = b
	self.db.char.colours.earthshield.a = a
	self:UpdateShieldBar()
end

function ShockAndAwe:getNoShieldColour(info)
	local colours = self.db.char.colours.noshield
	return colours.r, colours.g, colours.b, 0.2 -- use out of combat alpha
end

function ShockAndAwe:setNoShieldColour(info,r,g,b,a)
	self.db.char.colours.noshield.r = r
	self.db.char.colours.noshield.g = g
	self.db.char.colours.noshield.b = b
	self.db.char.colours.noshield.a = a
	self:UpdateShieldBar()
end

function ShockAndAwe:getEarthShockColour(info)
	local colours = self.db.char.colours.earthshock
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setEarthShockColour(info,r,g,b,a)
	self.db.char.colours.earthshock.r = r
	self.db.char.colours.earthshock.g = g
	self.db.char.colours.earthshock.b = b
	self.db.char.colours.earthshock.a = a
	self:ShockBar(self.db.char.colours.earthshock)
end

function ShockAndAwe:getFlameShockColour(info)
	local colours = self.db.char.colours.flameshock
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setFlameShockColour(info,r,g,b,a)
	self.db.char.colours.flameshock.r = r
	self.db.char.colours.flameshock.g = g
	self.db.char.colours.flameshock.b = b
	self.db.char.colours.flameshock.a = a
	self:ShockBar(self.db.char.colours.flameshock)
end

function ShockAndAwe:getFrostShockColour(info)
	local colours = self.db.char.colours.frostshock
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setFrostShockColour(info,r,g,b,a)
	self.db.char.colours.frostshock.r = r
	self.db.char.colours.frostshock.g = g
	self.db.char.colours.frostshock.b = b
	self.db.char.colours.frostshock.a = a
	self:ShockBar(self.db.char.colours.frostshock)
end

function ShockAndAwe:getWindShockColour(info)
	local colours = self.db.char.colours.windshock
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setWindShockColour(info,r,g,b,a)
	self.db.char.colours.windshock.r = r
	self.db.char.colours.windshock.g = g
	self.db.char.colours.windshock.b = b
	self.db.char.colours.windshock.a = a
	self:ShockBar(self.db.char.colours.windshock)
end

function ShockAndAwe:getMaelstromColour(info)
	local colours = self.db.char.colours.maelstrom
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setMaelstromColour(info,r,g,b,a)
	self.db.char.colours.maelstrom.r = r
	self.db.char.colours.maelstrom.g = g
	self.db.char.colours.maelstrom.b = b
	self.db.char.colours.maelstrom.a = a
	self:MaelstromBar()
end

function ShockAndAwe:getStormstrikeColour(info)
	local colours = self.db.char.colours.stormstrike
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setStormstrikeColour(info,r,g,b,a)
	self.db.char.colours.stormstrike.r = r
	self.db.char.colours.stormstrike.g = g
	self.db.char.colours.stormstrike.b = b
	self.db.char.colours.stormstrike.a = a
	self:StormstrikeBar()
end

function ShockAndAwe:getWindfuryColour(info)
	local colours = self.db.char.colours.windfury
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setWindfuryColour(info,r,g,b,a)
	self.db.char.colours.windfury.r = r
	self.db.char.colours.windfury.g = g
	self.db.char.colours.windfury.b = b
	self.db.char.colours.windfury.a = a
	self:WFProcBar()
end

function ShockAndAwe:getLavaLashColour(info)
	local colours = self.db.char.colours.lavalash
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setLavaLashColour(info,r,g,b,a)
	self.db.char.colours.lavalash.r = r
	self.db.char.colours.lavalash.g = g
	self.db.char.colours.lavalash.b = b
	self.db.char.colours.lavalash.a = a
	self:LavaLashBar()
end

function ShockAndAwe:getFeralSpiritCDColour(info)
	local colours = self.db.char.colours.feralspiritCD
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setFeralSpiritCDColour(info,r,g,b,a)
	self.db.char.colours.feralspiritCD.r = r
	self.db.char.colours.feralspiritCD.g = g
	self.db.char.colours.feralspiritCD.b = b
	self.db.char.colours.feralspiritCD.a = a
	local colours = self.db.char.colours.feralspiritCD
	self.frames["FeralSpiritCD"].statusbar:SetStatusBarColor(colours.r, colours.g, colours.b, colours.a)
end

function ShockAndAwe:getFeralSpiritColour(info)
	local colours = self.db.char.colours.feralspirit
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setFeralSpiritColour(info,r,g,b,a)
	self.db.char.colours.feralspirit.r = r
	self.db.char.colours.feralspirit.g = g
	self.db.char.colours.feralspirit.b = b
	self.db.char.colours.feralspirit.a = a
	local colours = self.db.char.colours.feralspirit
	self.frames["FeralSpirit"].statusbar:SetStatusBarColor(colours.r, colours.g, colours.b, colours.a)
end

function ShockAndAwe:getLavaBurstColour(info)
	local colours = self.db.char.colours.lavaburst
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setLavaBurstColour(info,r,g,b,a)
	self.db.char.colours.lavaburst.r = r
	self.db.char.colours.lavaburst.g = g
	self.db.char.colours.lavaburst.b = b
	self.db.char.colours.lavaburst.a = a
	self:LavaBurstBar()
end

function ShockAndAwe:getFlurryColour(info)
	local colours = self.db.char.uptime.flurry
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setFlurryColour(info,r,g,b,a)
	self.db.char.uptime.flurry.r = r
	self.db.char.uptime.flurry.g = g
	self.db.char.uptime.flurry.b = b
	self.db.char.uptime.flurry.a = a
	local buffColours = self.db.char.uptime.flurry
	self.uptime.session.buffs[Flurry].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
	self.uptime.lastfight.buffs[Flurry].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
end

function ShockAndAwe:getEDColour(info)
	local colours = self.db.char.uptime.ed
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setEDColour(info,r,g,b,a)
	self.db.char.uptime.ed.r = r
	self.db.char.uptime.ed.g = g
	self.db.char.uptime.ed.b = b
	self.db.char.uptime.ed.a = a
	local buffColours = self.db.char.uptime.ed
	self.uptime.session.buffs[ElementalDevastation].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
	self.uptime.lastfight.buffs[ElementalDevastation].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
end

function ShockAndAwe:getURColour(info)
	local colours = self.db.char.uptime.ur
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setURColour(info,r,g,b,a)
	self.db.char.uptime.ur.r = r
	self.db.char.uptime.ur.g = g
	self.db.char.uptime.ur.b = b
	self.db.char.uptime.ur.a = a
	local buffColours = self.db.char.uptime.ur
	self.uptime.session.buffs[UnleashedRage].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
	self.uptime.lastfight.buffs[UnleashedRage].barFrame.statusbar:SetStatusBarColor(buffColours.r, buffColours.g, buffColours.b, buffColours.a)
end

function ShockAndAwe:getWFTotalsColour(info)
	local colours = self.db.char.uptime.ur
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setWFTotalsColour(info,r,g,b,a)
	self.db.char.stats.wfcol.r = r
	self.db.char.stats.wfcol.g = g
	self.db.char.stats.wfcol.b = b
	self.db.char.stats.wfcol.a = a
end

-----------------------------------------
-- Priority Choices
-----------------------------------------

function ShockAndAwe:GetFirstPriority(info)
	return self.db.char.priority.option[1] 
end

function ShockAndAwe:SetFirstPriority(info, priorityValue)
	self.db.char.priority.option[1] = priorityValue
end

function ShockAndAwe:GetSecondPriority(info)
	return self.db.char.priority.option[2] 
end

function ShockAndAwe:SetSecondPriority(info, priorityValue)
	self.db.char.priority.option[2] = priorityValue
end

function ShockAndAwe:GetThirdPriority(info)
	return self.db.char.priority.option[3] 
end

function ShockAndAwe:SetThirdPriority(info, priorityValue)
	self.db.char.priority.option[3] = priorityValue
end

function ShockAndAwe:GetFourthPriority(info)
	return self.db.char.priority.option[4] 
end

function ShockAndAwe:SetFourthPriority(info, priorityValue)
	self.db.char.priority.option[4] = priorityValue
end

function ShockAndAwe:GetFifthPriority(info)
	return self.db.char.priority.option[5] 
end

function ShockAndAwe:SetFifthPriority(info, priorityValue)
	self.db.char.priority.option[5] = priorityValue
end

function ShockAndAwe:GetSixthPriority(info)
	return self.db.char.priority.option[6] 
end

function ShockAndAwe:SetSixthPriority(info, priorityValue)
	self.db.char.priority.option[6] = priorityValue
end

function ShockAndAwe:priorityQuery()
	return self.db.char.priority.show
end

function ShockAndAwe:ActivatePriority()
	self.db.char.priority.show = not self.db.char.priority.show
	if (self.db.char.priority.show) then
		self:Print(L["config_priority_on"])
	else
		self.PriorityFrame:Hide()
		self:Print(L["config_priority_off"])
	end
end

function ShockAndAwe:priorityTitleQuery()
	return self.db.char.priority.titleshow
end

function ShockAndAwe:ActivatePriorityTitle()
	self.db.char.priority.titleshow = not self.db.char.priority.titleshow
	if self.db.char.priority.titleshow then
		self.PriorityFrame.topText:Show()
		self:Print(L["config_prioritytitle_on"])
	else
		self.PriorityFrame.topText:Hide()
		self:Print(L["config_prioritytitle_off"])
	end
end

---------------
-- Warnings
---------------

function ShockAndAwe:WarningFrameQuery()
	return self.db.char.warning.show
end

function ShockAndAwe:ActivateWarningFrame()
	self.db.char.warning.show = not self.db.char.warning.show
	if (self.db.char.warning.show) then
		self:Print(L["config_warnframe_on"])
	else
		self:Print(L["config_warnframe_off"])
	end
end

function ShockAndAwe:shieldWarnQuery()
	return self.db.char.warning.shield
end

function ShockAndAwe:ActivateShieldWarn()
	self.db.char.warning.shield = not self.db.char.warning.shield
	if (self.db.char.warning.shield) then
		self:Print(L["config_shieldwarn_on"])
	else
		self:Print(L["config_shieldwarn_off"])
	end
end

function ShockAndAwe:weaponWarnQuery()
	return self.db.char.warning.weapon
end

function ShockAndAwe:ActivateWeaponWarn()
	self.db.char.warning.weapon = not self.db.char.warning.weapon
	if (self.db.char.warning.weapon) then
		self:Print(L["config_weaponwarn_on"])
	else
		self:Print(L["config_weaponwarn_off"])
	end
end

function ShockAndAwe:rangeWarnQuery()
	return self.db.char.warning.range
end

function ShockAndAwe:ActivateRangeWarn()
	self.db.char.warning.range = not self.db.char.warning.range
	if (self.db.char.warning.range) then
		self:Print(L["config_rangewarn_on"])
	else
		self:Print(L["config_rangewarn_off"])
	end
end

function ShockAndAwe:GetMainHandImbue(info)
	return self.db.char.binding.mhspell 
end

function ShockAndAwe:SetMainHandImbue(info, priorityValue)
	self.db.char.binding.mhspell = priorityValue
end

function ShockAndAwe:GetOffHandImbue(info)
	return self.db.char.binding.ohspell 
end

function ShockAndAwe:SetOffHandImbue(info, priorityValue)
	self.db.char.binding.ohspell = priorityValue
end

function ShockAndAwe:getWarningColour(info)
	local colours = self.db.char.warning.colour
	return colours.r, colours.g, colours.b, colours.a
end

function ShockAndAwe:setWarningColour(info,r,g,b,a)
	self.db.char.warning.colour.r = r
	self.db.char.warning.colour.g = g
	self.db.char.warning.colour.b = b
	self.db.char.warning.colour.a = a
end

function ShockAndAwe:GetWarningDuration(info)
	return self.db.char.warning.duration 
end

function ShockAndAwe:SetWarningDuration(info, priorityValue)
	self.db.char.warning.duration = priorityValue
end

function ShockAndAwe:OpenConfig()
	  if InterfaceOptionsFrame_OpenToCategory then
	    InterfaceOptionsFrame_OpenToCategory("ShockAndAwe");
    else
        InterfaceOptionsFrame_OpenToFrame("ShockAndAwe");
    end
end